# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2021 CERN.
# Copyright (C)      2022 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Celery tasks for fixtures."""

from datetime import datetime

from celery import shared_task
from invenio_access.permissions import system_identity
from invenio_pidstore.errors import (
    PIDAlreadyExists,
    PIDDeletedError,
    PIDDoesNotExistError,
)
from invenio_records_resources.services.uow import RecordIndexOp, unit_of_work
from invenio_search.engine import dsl

from ..proxies import current_communities


@shared_task
def create_demo_community(data, logo_path=None, feature=False):
    """Create a demo community."""
    service = current_communities.service
    try:
        community = service.create(data=data, identity=system_identity)

        # upload logo for community if provided
        if logo_path:
            with open(logo_path, "rb") as filestream:
                service.update_logo(system_identity, community.id, filestream)

        if feature:
            featured_data = {"start_date": datetime.utcnow().isoformat()}
            service.featured_create(system_identity, community.id, featured_data)

    except PIDAlreadyExists:
        pass


@shared_task
def reindex_featured_entries():
    """Reindexes records having at least one future entry which is now in the past."""
    service = current_communities.service
    now = datetime.utcnow().isoformat()

    @unit_of_work()
    def reindex_community(hit, uow=None):
        community = service.record_cls.pid.resolve(hit["id"])
        uow.register(
            RecordIndexOp(community, indexer=service.indexer, index_refresh=True)
        )

    # using scan to get all communities
    record_list = service.scan(
        identity=system_identity,
        extra_filter=dsl.Q("range", **{"featured.future": {"lte": now}}),
    )
    for hit in record_list.hits:
        try:
            reindex_community(hit)
        # community has been renamed or deleted in the meantime.
        # will be reindexed in the service method.
        except PIDDoesNotExistError:
            pass
        except PIDDeletedError:
            pass



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024 CERN.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.
"""Subcommunity request implementation."""

from invenio_access.permissions import system_identity
from invenio_i18n import lazy_gettext as _
from invenio_notifications.services.uow import NotificationOp
from invenio_requests.customizations import RequestType, actions
from marshmallow.exceptions import ValidationError

import invenio_communities.notifications.builders as notifications
from invenio_communities.proxies import current_communities

from ...notifications.builders import (
    SubComInvCommentNotificationBuilder,
    SubComReqCommentNotificationBuilder,
)


class AcceptSubcommunity(actions.AcceptAction):
    """Represents an accept action used to accept a subcommunity."""

    def execute(self, identity, uow):
        """Execute approve action."""
        to_be_moved = self.request.topic.resolve().id
        move_to = self.request.receiver.resolve().id
        current_communities.service.bulk_update_parent(
            system_identity, [to_be_moved], parent_id=move_to, uow=uow
        )
        uow.register(
            NotificationOp(
                notifications.SubCommunityAccept.build(
                    identity=identity, request=self.request
                )
            )
        )
        super().execute(identity, uow)


class DeclineSubcommunity(actions.DeclineAction):
    """Represents a decline action used to decline a subcommunity."""

    def execute(self, identity, uow):
        """Execute decline action."""
        # We override just to send a notification
        uow.register(
            NotificationOp(
                notifications.SubCommunityDecline.build(
                    identity=identity, request=self.request
                )
            )
        )
        super().execute(identity, uow)


class SubCommunityRequest(RequestType):
    """Request to join a parent community as a subcommunity."""

    type_id = "subcommunity"
    name = _("Subcommunity request")

    creator_can_be_none = False
    topic_can_be_none = False
    allowed_creator_ref_types = ["community"]
    allowed_receiver_ref_types = ["community"]
    allowed_topic_ref_types = ["community"]

    comment_notification_builder = SubComReqCommentNotificationBuilder

    available_actions = {
        "delete": actions.DeleteAction,
        "create": actions.CreateAndSubmitAction,
        "cancel": actions.CancelAction,
        # Custom implemented actions
        "accept": AcceptSubcommunity,
        "decline": DeclineSubcommunity,
    }

    needs_context = {
        "community_roles": [
            "owner",
            "manager",
        ]
    }


class CreateSubcommunityInvitation(actions.CreateAndSubmitAction):
    """Represents an accept action used to accept a subcommunity."""

    def execute(self, identity, uow):
        """Execute approve action."""
        parent = self.request.created_by.resolve()
        if not parent.children.allow:
            raise ValidationError("Assigned parent is not allowed to be a parent.")

        uow.register(
            NotificationOp(
                notifications.SubComInvitationCreate.build(
                    identity=identity, request=self.request
                )
            )
        )

        super().execute(identity, uow)


class AcceptSubcommunityInvitation(actions.AcceptAction):
    """Represents an accept action used to accept a subcommunity."""

    def execute(self, identity, uow):
        """Execute approve action."""
        child = self.request.receiver.resolve().id
        parent = self.request.created_by.resolve().id
        current_communities.service.bulk_update_parent(
            system_identity, [child], parent_id=parent, uow=uow
        )
        uow.register(
            NotificationOp(
                notifications.SubComInvitationAccept.build(
                    identity=identity, request=self.request
                )
            )
        )
        super().execute(identity, uow)


class DeclineSubcommunityInvitation(actions.DeclineAction):
    """Represents a decline action used to decline a subcommunity."""

    def execute(self, identity, uow):
        """Execute decline action."""
        # We override just to send a notification
        uow.register(
            NotificationOp(
                notifications.SubComInvitationDecline.build(
                    identity=identity, request=self.request
                )
            )
        )
        super().execute(identity, uow)


class SubCommunityInvitationRequest(RequestType):
    """Request from a parent community to community to join."""

    type_id = "subcommunity-invitation"
    name = _("Subcommunity invitation")

    creator_can_be_none = False
    topic_can_be_none = True
    allowed_creator_ref_types = ["community"]
    allowed_receiver_ref_types = ["community"]
    allowed_topic_ref_types = ["community"]

    comment_notification_builder = SubComInvCommentNotificationBuilder

    available_actions = {
        "delete": actions.DeleteAction,
        "cancel": actions.CancelAction,
        # Custom implemented actions
        "create": CreateSubcommunityInvitation,
        "accept": AcceptSubcommunityInvitation,
        "decline": DeclineSubcommunityInvitation,
    }

    needs_context = {
        "community_roles": [
            "owner",
            "manager",
        ]
    }


def subcommunity_request_type(app):
    """Return the subcommunity request type.

    Since it can be overridden by the application, this function should be used
    as the entry point for the request type.

    It must return a class that inherits from `RequestType`.
    """
    if not app:
        return
    return app.config.get("COMMUNITIES_SUB_REQUEST_CLS", SubCommunityRequest)


def subcommunity_invitation_request_type(app):
    """Return the subcommunity request type.

    Since it can be overridden by the application, this function should be used
    as the entry point for the request type.

    It must return a class that inherits from `RequestType`.
    """
    if not app:
        return
    return app.config.get(
        "COMMUNITIES_SUB_INVITATION_REQUEST_CLS", SubCommunityInvitationRequest
    )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024 CERN.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.
"""Subcommunity resource."""

from .config import SubCommunityResourceConfig
from .resource import SubCommunityResource

__all__ = ("SubCommunityResource", "SubCommunityResourceConfig")



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2023-2024 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Children system field."""

from invenio_records.systemfields import SystemField


class Children:
    """Children object."""

    def __init__(self, allow=None):
        """Create a Children object."""
        self.dirty = allow is not None
        self._allow = allow or False

    @property
    def allow(self):
        """Get the allow."""
        return self._allow

    @allow.setter
    def allow(self, value):
        """Set the allow."""
        if not isinstance(value, bool):
            raise ValueError("Invalid value for allow, it must be a boolean.")
        self._allow = value
        self.dirty = True

    @classmethod
    def from_dict(cls, data):
        """Create a Children object from a dictionary."""
        return cls(allow=data.get("allow"))

    def dump(self):
        """Dump the Children object to a dictionary."""
        return {"allow": self.allow}


class ChildrenField(SystemField):
    """System field for children of a community."""

    children_obj_class = Children

    def __init__(self, key="children", children_obj_class=None):
        """Create a new ChildrenField instance."""
        self._children_obj_class = children_obj_class or self.children_obj_class
        super().__init__(key=key)

    def obj(self, instance):
        """Get the Children object."""
        obj = self._get_cache(instance)
        if obj is not None:
            return obj

        data = self.get_dictkey(instance)
        if data:
            obj = self._children_obj_class.from_dict(data)
        else:
            obj = self._children_obj_class()

        self._set_cache(instance, obj)
        return obj

    def set_obj(self, record, obj):
        """Set the Children object."""
        # We accept both dicts and Children class objects.
        if isinstance(obj, dict):
            obj = self._children_obj_class.from_dict(obj)

        if not isinstance(obj, self._children_obj_class):
            raise ValueError("Invalid children object.")

        # We do not dump the object until the pre_commit hook
        self._set_cache(record, obj)

    def __get__(self, record, owner=None):
        """Get the record's Children object."""
        if record is None:
            # access by class
            return self

        # access by object
        return self.obj(record)

    def __set__(self, record, obj):
        """Set the records Children object."""
        self.set_obj(record, obj)

    def pre_commit(self, record):
        """Dump the configured values before the record is committed."""
        obj = self.obj(record)
        if obj.dirty:
            record[self.key] = obj.dump()



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2022-2024 CERN.
# Copyright (C) 2023 TU Wien.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""UI community schema."""

from functools import partial

from flask import g
from flask_resources import BaseObjectSchema
from idutils import detect_identifier_schemes, to_url
from invenio_i18n import get_locale
from invenio_i18n import lazy_gettext as _
from invenio_records_resources.services.custom_fields import CustomFieldsSchemaUI
from invenio_vocabularies.contrib.awards.serializer import AwardL10NItemSchema
from invenio_vocabularies.contrib.funders.serializer import FunderL10NItemSchema
from invenio_vocabularies.resources import VocabularyL10Schema
from marshmallow import Schema, fields, post_dump
from marshmallow_utils.fields import FormatEDTF as FormatEDTF_

from invenio_communities.communities.schema import CommunityThemeSchema
from invenio_communities.proxies import current_communities


def _community_permission_check(action, community, identity):
    """Check community permission for identity."""
    try:
        community_id = getattr(community, "id", community["id"])
    except KeyError:
        community_id = getattr(
            community["processed"][0],
            "community_id",
            community["processed"][0]["community_id"],
        )

    return current_communities.service.config.permission_policy_cls(
        action,
        community_id=community_id,
        record=community,
    ).allows(identity)


def mask_removed_by(obj):
    """Mask information about who removed the community."""
    return_value = _("Unknown")
    removed_by = obj.get("removed_by", None)

    if removed_by is not None:
        user = removed_by.get("user", None)

        if user == "system":
            return_value = _("System (automatic)")
        elif user is not None:
            return_value = _("Admin")

    return return_value


# Partial to make short definitions in below schema.
FormatEDTF = partial(FormatEDTF_, locale=get_locale)


class TombstoneSchema(Schema):
    """Schema for a record tombstone."""

    removal_reason = fields.Nested(VocabularyL10Schema, attribute="removal_reason")

    note = fields.String(attribute="note")

    removed_by = fields.Function(mask_removed_by)

    removal_date_l10n_medium = FormatEDTF(attribute="removal_date", format="medium")

    removal_date_l10n_long = FormatEDTF(attribute="removal_date", format="long")

    citation_text = fields.String(attribute="citation_text")

    is_visible = fields.Boolean(attribute="is_visible")


class FundingSchema(Schema):
    """Schema for dumping types in the UI."""

    award = fields.Nested(AwardL10NItemSchema)
    funder = fields.Nested(FunderL10NItemSchema)


class UICommunitySchema(BaseObjectSchema):
    """Schema for dumping extra information of the community for the UI."""

    type = fields.Nested(VocabularyL10Schema, attribute="metadata.type")

    funding = fields.List(
        fields.Nested(FundingSchema()),
        attribute="metadata.funding",
    )

    tombstone = fields.Nested(TombstoneSchema, attribute="tombstone")

    organizations = fields.Method("get_organizations", dump_only=True)

    theme = fields.Nested(CommunityThemeSchema, dump_only=True, load_default={})

    # Custom fields
    custom_fields = fields.Nested(
        partial(CustomFieldsSchemaUI, fields_var="COMMUNITIES_CUSTOM_FIELDS")
    )

    permissions = fields.Method("get_permissions", dump_only=True)

    def get_organizations(self, obj):
        """Get organizations with metadata."""
        organizations = obj.get("metadata", {}).get("organizations", [])

        for org in organizations:
            if "id" in org:
                scheme = detect_identifier_schemes(org["id"])[0]
                org["icon"] = f"/static/images/{scheme}-icon.svg"
                org["url"] = to_url(org["id"], scheme)
                org["label"] = scheme

        return organizations

    def get_permissions(self, obj):
        """Get permission."""
        if obj == {}:
            return {}

        can_include_directly = _community_permission_check(
            "include_directly", community=obj, identity=g.identity
        )
        can_update = _community_permission_check(
            "update", community=obj, identity=g.identity
        )
        can_submit_record = _community_permission_check(
            "submit_record", community=obj, identity=g.identity
        )
        return {
            "can_include_directly": can_include_directly,
            "can_update": can_update,
            "can_submit_record": can_submit_record,
        }

    @post_dump(pass_original=True)
    def post_dump(self, data, original, many, **kwargs):
        """Pop tombstone field if not deleted/visible."""
        is_deleted = (original.get("deletion_status") or {}).get("is_deleted", False)
        tombstone_visible = (original.get("tombstone") or {}).get("is_visible", True)

        if not is_deleted or not tombstone_visible:
            data.pop("tombstone", None)

        return data


class TypesSchema(Schema):
    """Schema for dumping types in the UI."""

    types = fields.List(fields.Nested(VocabularyL10Schema), attribute="types")



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2024 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Decorators."""

from functools import wraps
from warnings import warn

from flask import g, request

from invenio_communities.communities.resources.serializer import (
    UICommunityJSONSerializer,
)
from invenio_communities.proxies import current_communities


def pass_community(serialize):
    """Fetch the community record."""

    def decorator(f):
        @wraps(f)
        def view(**kwargs):
            pid_value = kwargs["pid_value"]
            community = current_communities.service.read(
                id_=pid_value, identity=g.identity
            )
            kwargs["community"] = community
            request.community = community.to_dict()
            if serialize:
                community_ui = UICommunityJSONSerializer().dump_obj(community.to_dict())
                kwargs["community_ui"] = community_ui

            return f(**kwargs)

        return view

    return decorator


def warn_deprecation(deprecated_route, new_route):
    """Decorator to log warnings for deprecated routes."""

    def decorator(f):
        @wraps(f)
        def view(**kwargs):
            warn(
                f"The '{deprecated_route}' route is deprecated and will be removed in future releases. "
                f"Please update to use '{new_route}' instead. If you want to keep using the old endpoint, then you can set up redirection separately.",
                DeprecationWarning,
                stacklevel=2,
            )
            return f(**kwargs)

        return view

    return decorator



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Northwestern University.
# Copyright (C) 2022 CERN.
# Copyright (C) 2023 Graz University of Technology.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Member schema."""

from datetime import timezone
from types import SimpleNamespace

from invenio_i18n import lazy_gettext as _
from invenio_users_resources.proxies import (
    current_groups_service,
    current_users_service,
)
from marshmallow import Schema, ValidationError, fields, validate, validates_schema
from marshmallow_utils.fields import SanitizedUnicode, TZDateTime

from .fields import RoleField


#
# Utility schemas
#
class MemberEntitySchema(Schema):
    """Represents a single member entity."""

    type = fields.String(
        required=True, validate=validate.OneOf(["user", "group", "email"])
    )
    id = fields.String(required=True)
    is_current_user = fields.Bool(dump_only=True)


class MembersSchema(Schema):
    """Members Schema."""

    members = fields.List(
        fields.Nested(MemberEntitySchema),
        # max is on purpose to limit the max number of additions/changes/
        # removals per request as they all run in a single transaction and
        # requires resources to hold.
        validate=validate.Length(min=1, max=100),
    )


class RequestSchema(Schema):
    """Request Schema."""

    id = fields.String()
    status = fields.String()
    is_open = fields.Boolean()
    # TODO: expires_at is dumped in the index and thus a string. This is
    # because the relations field doesn't properly load data from the index
    # (it should have converted expires_at into a datetime object).
    expires_at = fields.String()


#
# Schemas used for validation
#
class AddBulkSchema(MembersSchema, Schema):
    """Schema used for adding members."""

    role = RoleField(required=True)
    visible = fields.Boolean()


class InviteBulkSchema(AddBulkSchema):
    """Schema used for inviting members."""

    message = SanitizedUnicode()


class UpdateBulkSchema(MembersSchema, Schema):
    """Schema used for updating members."""

    role = RoleField()
    visible = fields.Boolean()

    @validates_schema
    def validate_schema(self, data, **kwargs):
        """Validates that role and/or visible is set."""
        if "role" not in data and "visible" not in data:
            raise ValidationError(_("Missing fields 'role' and/or 'visible'."))


class DeleteBulkSchema(MembersSchema):
    """Delete bulk schema."""


class RequestMembershipSchema(Schema):
    """Schema used for requesting membership."""

    message = SanitizedUnicode()


#
# Schemas used for dumping a single member
#


class PublicDumpSchema(Schema):
    """Public Dump Schema."""

    id = fields.String(required=True)
    member = fields.Method("get_member")

    def get_member(self, obj):
        """Get a member."""
        if obj.user_id:
            return self.get_user_member(obj["user"])
        elif obj.group_id:
            return self.get_group_member(obj["group"])

    def get_user_member(self, user):
        """Get a user member."""
        profile = user.get("profile", {})
        name = profile.get("full_name") or user.get("username") or _("Untitled")
        description = profile.get("affiliations") or ""
        fake_user_obj = SimpleNamespace(id=user["id"])
        current_identity = self.context["identity"]
        avatar = current_users_service.links_item_tpl.expand(
            current_identity, fake_user_obj
        )["avatar"]

        return {
            "type": "user",
            "id": user["id"],
            "name": name,
            "description": description,
            "avatar": avatar,
        }

    def get_group_member(self, group):
        """Get a group member."""
        fake_group_obj = SimpleNamespace(id=group["id"])
        current_identity = self.context["identity"]
        avatar = current_groups_service.links_item_tpl.expand(
            current_identity, fake_group_obj
        )["avatar"]
        return {
            "type": "group",
            "id": group["id"],
            "name": group.get("name") or group["id"],
            "description": group.get("description", ""),
            "avatar": avatar,
        }


class MemberDumpSchema(PublicDumpSchema):
    """Schema for dumping members."""

    role = fields.String()
    visible = fields.Boolean()

    is_current_user = fields.Method("get_current_user")
    permissions = fields.Method("get_permissions")

    created = TZDateTime(timezone=timezone.utc, format="iso")
    updated = TZDateTime(timezone=timezone.utc, format="iso")
    revision_id = fields.Integer()

    def is_self(self, obj):
        """Get permission."""
        if "is_self" not in self.context:
            current_identity = self.context["identity"]
            self.context["is_self"] = (
                obj.user_id is not None
                and current_identity.id is not None
                and str(obj.user_id) == str(current_identity.id)
            )
        return self.context["is_self"]

    def get_current_user(self, obj):
        """Get permission."""
        return self.is_self(obj)

    def get_permissions(self, obj):
        """Get permission."""
        permission_check = self.context["field_permission_check"]

        # Does not take CommunitySelfMember into account because no "member" is
        # passed to the permission check.
        can_update = permission_check(
            "members_update",
            community_id=obj.community_id,
            role=obj.role,
        )
        is_self = self.is_self(obj)

        # The rules below are defined by the update()/delete() methods in the
        # service.
        return {
            "can_leave": is_self,
            "can_delete": can_update and not is_self,
            "can_update_role": can_update and not is_self,
            "can_update_visible": (obj.visible and can_update) or is_self,
        }


class InvitationDumpSchema(MemberDumpSchema):
    """Schema for dumping invitations."""

    request = fields.Nested(RequestSchema)
    permissions = fields.Method("get_permissions")

    def get_permissions(self, obj):
        """Get permission."""
        # Only owners and managers can list invitations, and thus only the
        # request status is necessary to determine if the identity can cancel.
        is_open = obj["request"]["status"] == "submitted"
        permission_check = self.context["field_permission_check"]

        return {
            "can_cancel": is_open,
            "can_update_role": is_open
            and permission_check(
                "members_update",
                community_id=obj.community_id,
                member=obj,
            ),
        }



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2024 CERN.
# Copyright (C) 2023-2024 Graz University of Technology.
# Copyright (C) 2023 KTH Royal Institute of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Communities UI views."""

from datetime import datetime

from babel.dates import format_datetime
from flask import Blueprint, current_app, g, render_template, request, url_for
from flask_login import current_user
from invenio_pidstore.errors import PIDDeletedError, PIDDoesNotExistError
from invenio_records_resources.proxies import current_service_registry
from invenio_records_resources.services.errors import PermissionDeniedError

from invenio_communities.communities.resources.serializer import (
    UICommunityJSONSerializer,
)
from invenio_communities.proxies import current_communities

from ..errors import CommunityDeletedError, LogoNotFoundError
from ..searchapp import search_app_context
from .communities import (
    communities_about,
    communities_curation_policy,
    communities_frontpage,
    communities_new,
    communities_new_subcommunity,
    communities_requests,
    communities_search,
    communities_settings,
    communities_settings_pages,
    communities_settings_privileges,
    communities_settings_submission_policy,
    communities_subcommunities,
    community_theme_css_config,
    invitations,
    members,
)
from .decorators import warn_deprecation


#
# Error handlers
#
def not_found_error(error):
    """Handler for 'Not Found' errors."""
    return render_template(current_app.config["THEME_404_TEMPLATE"]), 404


def record_tombstone_error(error):
    """Tombstone page."""
    record = getattr(error, "record", None)
    if (record_ui := getattr(error, "result_item", None)) is not None:
        if record is None:
            record = record_ui._record
        record_ui = UICommunityJSONSerializer().dump_obj(record_ui.to_dict())

    # render a 404 page if the tombstone isn't visible
    if not record.tombstone.is_visible:
        return not_found_error(error)

    # we only render a tombstone page if there is a record with a visible tombstone
    return (
        render_template(
            "invenio_communities/tombstone.html",
            record=record_ui,
        ),
        410,
    )


def record_permission_denied_error(error):
    """Handle permission denier error on record views."""
    if not current_user.is_authenticated:
        # trigger the flask-login unauthorized handler
        return current_app.login_manager.unauthorized()
    return render_template(current_app.config["THEME_403_TEMPLATE"]), 403


def _can_create_community():
    """Function used to check if a user has permissions to create a community."""
    return current_communities.service.check_permission(g.identity, "create")


def _show_create_community_link():
    """
    Determine if the 'New community' button should always be visible.

    If the 'COMMUNITIES_ALWAYS_SHOW_CREATE_LINK' config is False,
    check the user's permission to create a community link. If the config is
    True, the button is always visible.
    """
    should_show = current_app.config.get("COMMUNITIES_ALWAYS_SHOW_CREATE_LINK", False)
    if not should_show:  # show only when user can create
        should_show = _can_create_community()
    return should_show


def _has_about_page_content():
    """Function used to check if about page has content."""
    community = request.community
    if community and "metadata" in community and "page" in community["metadata"]:
        return community["metadata"]["page"] != ""
    return False


def _has_curation_policy_page_content():
    """Function used to check if curation policy page has content."""
    community = request.community
    if (
        community
        and "metadata" in community
        and "curation_policy" in community["metadata"]
    ):
        return community["metadata"]["curation_policy"] != ""
    return False


#
# Registration
#
def create_ui_blueprint(app):
    """Register blueprint routes on app."""
    routes = app.config.get("COMMUNITIES_ROUTES")

    blueprint = Blueprint(
        "invenio_communities",
        __name__,
        template_folder="../templates",
        static_folder="../static",
    )

    # Communities URL rules
    blueprint.add_url_rule(
        routes["frontpage"],
        view_func=communities_frontpage,
        strict_slashes=False,
    )

    blueprint.add_url_rule(
        routes["search"],
        view_func=communities_search,
        strict_slashes=False,
    )

    blueprint.add_url_rule(
        routes["deprecated_search"],
        endpoint="deprecated_communities_search",
        view_func=warn_deprecation(routes["deprecated_search"], routes["search"])(
            communities_search
        ),
        strict_slashes=False,
    )

    blueprint.add_url_rule(
        routes["new"],
        view_func=communities_new,
    )

    blueprint.add_url_rule(
        routes["deprecated_new"],
        endpoint="deprecated_communities_new",
        view_func=warn_deprecation(routes["deprecated_new"], routes["new"])(
            communities_new
        ),
    )

    blueprint.add_url_rule(
        routes["about"],
        view_func=communities_about,
    )

    blueprint.add_url_rule(
        routes["curation_policy"],
        view_func=communities_curation_policy,
    )

    blueprint.add_url_rule(
        routes["new_subcommunity"],
        view_func=communities_new_subcommunity,
    )

    blueprint.add_url_rule(
        routes["subcommunities"],
        view_func=communities_subcommunities,
    )

    # Settings tab routes
    blueprint.add_url_rule(
        routes["settings"],
        view_func=communities_settings,
    )

    blueprint.add_url_rule(
        routes["requests"],
        view_func=communities_requests,
    )

    blueprint.add_url_rule(
        routes["settings_privileges"],
        view_func=communities_settings_privileges,
    )

    blueprint.add_url_rule(
        routes["settings_submission_policy"],
        view_func=communities_settings_submission_policy,
    )

    blueprint.add_url_rule(
        routes["settings_pages"],
        view_func=communities_settings_pages,
    )

    blueprint.add_url_rule(routes["members"], view_func=members)

    blueprint.add_url_rule(routes["invitations"], view_func=invitations)

    # theme injection view
    blueprint.add_url_rule(
        "/communities/<pid_value>/community-theme-<revision>.css",
        view_func=community_theme_css_config,
    )

    # Register error handlers
    blueprint.register_error_handler(
        PermissionDeniedError, record_permission_denied_error
    )
    blueprint.register_error_handler(CommunityDeletedError, record_tombstone_error)
    blueprint.register_error_handler(PIDDeletedError, record_tombstone_error)
    blueprint.register_error_handler(PIDDoesNotExistError, not_found_error)

    # Register context processor
    blueprint.app_context_processor(search_app_context)

    # Template filters
    @blueprint.app_template_filter()
    def invenio_format_datetime(value):
        date = datetime.fromisoformat(value)
        locale_value = current_app.config.get("BABEL_DEFAULT_LOCALE")
        return format_datetime(date, locale=locale_value)

    @blueprint.app_template_filter("resolve_community_logo")
    def resolve_community_logo(logo_link, community_id):
        """Returns placeholder image link if passed community doesn't have a logo."""
        community_service = current_service_registry.get("communities")

        try:
            community_service.read_logo(
                identity=g.identity,
                id_=community_id,
            )
        except LogoNotFoundError:
            return url_for("static", filename="images/square-placeholder.png")

        return logo_link

    return blueprint



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024 CERN.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.
"""User moderation tasks."""

from celery import shared_task
from invenio_access.permissions import system_identity

from invenio_communities.errors import DeletionStatusError
from invenio_communities.proxies import current_communities


@shared_task(ignore_result=True)
def delete_community(community_id, tombstone_data):
    """Delete a single community."""
    try:
        current_communities.service.delete_community(
            system_identity, community_id, tombstone_data
        )
    except DeletionStatusError as ex:
        # Community is already deleted; index again to make sure search is up-to-date.
        current_communities.indexer.index(ex.community)


@shared_task(ignore_result=True)
def restore_community(community_id):
    """Restore a single community."""
    try:
        current_communities.service.restore_community(system_identity, community_id)
    except DeletionStatusError as ex:
        # Community is already restored; index again to make sure search is up-to-date.
        current_communities.indexer.index(ex.community)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 TU Wien.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""System field for managing the community deletion status."""

import enum

from invenio_records.systemfields import SystemField


class CommunityDeletionStatusEnum(enum.Enum):
    """Enumeration of a community's possible deletion states."""

    PUBLISHED = "P"
    DELETED = "D"
    MARKED = "X"


class CommunityDeletionStatus:
    """The deletion status of the community."""

    def __init__(self, status):
        """Constructor."""
        self.status = status

    @property
    def status(self):
        """Get the deletion status."""
        return self._status.value

    @status.setter
    def status(self, value):
        """Set the deletion status."""
        if value is None:
            self._status = CommunityDeletionStatusEnum.PUBLISHED

        elif isinstance(value, str):
            self._status = CommunityDeletionStatusEnum(value)

        elif isinstance(value, CommunityDeletionStatusEnum):
            self._status = value

        else:
            raise ValueError(f"Invalid value for community deletion status: {value}")

    @property
    def is_deleted(self):
        """Check if the community is deleted."""
        return self._status != CommunityDeletionStatusEnum.PUBLISHED

    def __repr__(self):
        """Return repr(self)."""
        return f"<CommunityDeletionStatus {self._status.name}: '{self._status.value}'>"

    def __str__(self):
        """Return str(self)."""
        return self.status

    def __eq__(self, other):
        """Check if self and other are equal.

        This allows checking against other instances of the same type, strings,
        and ``CommunityDeletionStatusEnum`` values.
        """
        if isinstance(other, type(self)):
            return self.status == other.status

        elif isinstance(other, CommunityDeletionStatusEnum):
            return self.status == other.value

        elif isinstance(other, str):
            return self.status == other

        return False


class CommunityDeletionStatusField(SystemField):
    """System field for accessing a community's deletion status."""

    #
    # Data descriptor methods (i.e. attribute access)
    #
    def __get__(self, record, owner=None):
        """Get the deletion status of the community."""
        if record is None:
            return self  # returns the field itself.

        status = self._get_cache(record) or CommunityDeletionStatus(
            record.model.deletion_status
        )
        self._set_cache(record, status)
        return status

    def __set__(self, record, value):
        """Set the deletion status of the community."""
        status = CommunityDeletionStatus(value)
        record.model.deletion_status = status._status
        self._set_cache(record, status)

    def pre_commit(self, record):
        """Dump the deletion status to the community before committing."""
        status = self._get_cache(record) or CommunityDeletionStatus(None)
        record.model.deletion_status = record.get("deletion_status", status._status)

    def pre_dump(self, record, data, **kwargs):
        """Dump the deletion status information."""
        status = CommunityDeletionStatus(record.model.deletion_status)
        # mitigation of deletion_status.is_deleted missing from the mapping
        # currently it is a string
        # don't confuse with record.model.is_deleted!
        data["is_deleted"] = status.is_deleted
        data["deletion_status"] = status.status

    def post_load(self, record, data, **kwargs):
        """After loading, set the deletion status."""
        deletion_status = data.get("deletion_status", None)
        self.__set__(record, deletion_status)
        # mitigation of deletion_status.is_deleted missing from the mapping
        # currently it is a string
        # don't confuse with record.model.is_deleted!
        record.pop("is_deleted", None)
        record.pop("deletion_status", None)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024 CERN.
# Copyright (C) 2023 Graz University of Technology.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Notification related utils for notifications."""

from invenio_notifications.models import Notification
from invenio_notifications.registry import EntityResolverRegistry
from invenio_notifications.services.builders import NotificationBuilder
from invenio_notifications.services.generators import EntityResolve, UserEmailBackend
from invenio_requests.notifications.filters import UserRecipientFilter
from invenio_users_resources.notifications.filters import UserPreferencesRecipientFilter
from invenio_users_resources.notifications.generators import UserRecipient

from invenio_communities.notifications.generators import CommunityMembersRecipient


class BaseNotificationBuilder(NotificationBuilder):
    """Base notification builder for community invitation action."""

    context = [
        EntityResolve(key="request"),
        EntityResolve(key="request.created_by"),
        EntityResolve(key="request.receiver"),
    ]

    recipient_filters = [
        UserPreferencesRecipientFilter(),
    ]

    recipient_backends = [
        UserEmailBackend(),
    ]


class CommunityInvitationNotificationBuilder(BaseNotificationBuilder):
    """Base notification builder for community invitation action."""


class CommunityInvitationSubmittedNotificationBuilder(
    CommunityInvitationNotificationBuilder
):
    """Notification builder for community invitation submit action."""

    type = "community-invitation.submit"

    @classmethod
    def build(cls, request, role, message=None):
        """Build notification with request context."""
        return Notification(
            type=cls.type,
            context={
                "request": EntityResolverRegistry.reference_entity(request),
                "role": role,
                "message": message,
            },
        )

    recipients = [
        UserRecipient(key="request.receiver"),
    ]


class CommunityInvitationAcceptNotificationBuilder(
    CommunityInvitationNotificationBuilder
):
    """Notification builder for community invitation accept action."""

    type = "community-invitation.accept"

    @classmethod
    def build(cls, request):
        """Build notification with request context."""
        return Notification(
            type=cls.type,
            context={
                "request": EntityResolverRegistry.reference_entity(request),
            },
        )

    recipients = [
        CommunityMembersRecipient(key="request.created_by", roles=["owner", "manager"]),
    ]


class CommunityInvitationCancelNotificationBuilder(
    CommunityInvitationNotificationBuilder
):
    """Notification builder for community invitation cancel action."""

    type = "community-invitation.cancel"

    @classmethod
    def build(cls, request):
        """Build notification with request context."""
        return Notification(
            type=cls.type,
            context={
                "request": EntityResolverRegistry.reference_entity(request),
            },
        )

    recipients = [
        UserRecipient(key="request.receiver"),
    ]


class CommunityInvitationDeclineNotificationBuilder(
    CommunityInvitationNotificationBuilder
):
    """Notification builder for community invitation decline action."""

    type = "community-invitation.decline"

    @classmethod
    def build(cls, request):
        """Build notification with request context."""
        return Notification(
            type=cls.type,
            context={
                "request": EntityResolverRegistry.reference_entity(request),
            },
        )

    recipients = [
        CommunityMembersRecipient(key="request.created_by", roles=["owner", "manager"]),
    ]


class CommunityInvitationExpireNotificationBuilder(
    CommunityInvitationNotificationBuilder
):
    """Notification builder for community invitation expire action."""

    type = "community-invitation.expire"

    @classmethod
    def build(cls, request):
        """Build notification with request context."""
        return Notification(
            type=cls.type,
            context={
                "request": EntityResolverRegistry.reference_entity(request),
            },
        )

    recipients = [
        CommunityMembersRecipient(key="request.created_by", roles=["owner", "manager"]),
        UserRecipient(key="request.receiver"),
    ]


#
# Subcommunity request
#
class SubCommunityBuilderBase(BaseNotificationBuilder):
    """Base notification builder for subcommunity requests."""

    type = "subcommunity-request"

    context = [
        EntityResolve("request"),
        EntityResolve("request.created_by"),
        EntityResolve("request.topic"),
        EntityResolve("request.receiver"),
        EntityResolve("executing_user"),
    ]

    @classmethod
    def build(cls, identity, request):
        """Build notification with request context."""
        return Notification(
            type=cls.type,
            context={
                "request": EntityResolverRegistry.reference_entity(request),
                "executing_user": EntityResolverRegistry.reference_identity(identity),
            },
        )

    recipients = [
        CommunityMembersRecipient("request.created_by", roles=["owner", "manager"]),
        CommunityMembersRecipient("request.receiver", roles=["owner", "manager"]),
    ]

    recipient_filters = [
        UserPreferencesRecipientFilter(),
        # Don't send notifications to the user performing the action
        UserRecipientFilter("executing_user"),
    ]


class SubCommunityCreate(SubCommunityBuilderBase):
    """Notification builder for subcommunity request creation."""

    type = f"{SubCommunityBuilderBase.type}.create"

    recipient_filters = [
        UserPreferencesRecipientFilter(),
        # TODO: We exceptionally DON'T filter out the executing user here, because we
        # don't have a clear place where they can see the created request.
        # See also: https://github.com/inveniosoftware/invenio-communities/issues/1158
        # UserRecipientFilter("executing_user"),
    ]


class SubCommunityAccept(SubCommunityBuilderBase):
    """Notification builder for subcommunity request accept."""

    type = f"{SubCommunityBuilderBase.type}.accept"


class SubCommunityDecline(SubCommunityBuilderBase):
    """Notification builder for subcommunity request decline."""

    type = f"{SubCommunityBuilderBase.type}.decline"


class SubComInvitationBuilderBase(SubCommunityBuilderBase):
    """Base notification builder for subcommunity invitation requests."""

    type = "subcommunity-invitation-request"

    context = [
        EntityResolve("request"),
        EntityResolve("request.created_by"),
        EntityResolve("request.receiver"),
        EntityResolve("executing_user"),
    ]


class SubComInvitationCreate(SubComInvitationBuilderBase):
    """Notification builder for subcommunity request creation."""

    type = f"{SubComInvitationBuilderBase.type}.create"

    context = [
        EntityResolve("request"),
        EntityResolve("request.created_by"),
        EntityResolve("request.receiver"),
        # EntityResolve("executing_user") creating via script only for now
    ]

    recipients = [
        CommunityMembersRecipient("request.receiver", roles=["owner", "manager"]),
    ]


class SubComInvitationAccept(SubComInvitationBuilderBase):
    """Notification builder for subcommunity request accept."""

    type = f"{SubComInvitationBuilderBase.type}.accept"

    recipient_filters = [
        UserPreferencesRecipientFilter(),
        # Don't send notifications to the user performing the action
        UserRecipientFilter("executing_user"),
    ]


class SubComInvitationDecline(SubComInvitationBuilderBase):
    """Notification builder for subcommunity request decline."""

    type = f"{SubComInvitationBuilderBase.type}.decline"

    recipient_filters = [
        UserPreferencesRecipientFilter(),
        # Don't send notifications to the user performing the action
        UserRecipientFilter("executing_user"),
    ]


class SubComInvitationExpire(SubComInvitationBuilderBase):
    """Notification builder for subcommunity invitation expire."""

    type = f"{SubComInvitationBuilderBase.type}.expire"

    context = [
        EntityResolve("request"),
        EntityResolve("request.created_by"),
        EntityResolve("request.receiver"),
    ]

    recipients = [
        CommunityMembersRecipient("request.receiver", roles=["owner", "manager"]),
    ]


#
# Comments
#
class SubComCommentNotificationBuilderBase(SubCommunityBuilderBase):
    """Notification builder for comment request event creation."""

    context = [
        EntityResolve(key="request"),
        EntityResolve(key="request.created_by"),
        EntityResolve(key="request.receiver"),
        EntityResolve(key="request_event"),
        EntityResolve(key="request_event.created_by"),
    ]

    @classmethod
    def build(cls, request, request_event):
        """Build notification with request context."""
        return Notification(
            type=cls.type,
            context={
                "request": EntityResolverRegistry.reference_entity(request),
                "request_event": EntityResolverRegistry.reference_entity(request_event),
            },
        )

    recipient_filters = [
        # do not send notification to user creating the comment
        UserRecipientFilter(key="request_event.created_by"),
        UserPreferencesRecipientFilter(),
    ]


class SubComReqCommentNotificationBuilder(SubComCommentNotificationBuilderBase):
    """Notification builder for comment request event creation."""

    type = f"comment-{SubCommunityBuilderBase.type}.create"


class SubComInvCommentNotificationBuilder(SubComCommentNotificationBuilderBase):
    """Notification builder for comment request event creation."""

    type = f"comment-{SubComInvitationBuilderBase.type}.create"
